// Copyright (c) 2021.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software
// and associated documentation files (the "Software"), to deal in the Software without restriction,
// including without limitation the rights to use, copy, modify, merge, publish, distribute,
// sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or
// substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
// THE USE OR OTHER DEALINGS IN THE SOFTWARE.

#include "IEncoder.hpp"

#include <QThread>

extern "C" {
#include <libavutil/avutil.h>
#include <libavutil/hwcontext.h>
#include <libavfilter/avfilter.h>
#include <libavformat/avformat.h>
#include <libavdevice/avdevice.h>
#include <libavcodec/avcodec.h>
#include <libswscale/swscale.h>
#include <libswresample/swresample.h>
#include <libavutil/imgutils.h>
}

#ifndef LIBAVQT_ENCODERQSV_H
#define LIBAVQT_ENCODERQSV_H

namespace AVQt {
    class EncoderQSVPrivate;

    class EncoderQSV : public QThread, public IEncoder {
    Q_OBJECT

        Q_DECLARE_PRIVATE(AVQt::EncoderQSV)

        Q_INTERFACES(AVQt::IEncoder)

    public:
        explicit EncoderQSV(CODEC codec, int bitrate, QObject *parent = nullptr);

        EncoderQSV(EncoderQSV &&other) noexcept;

        explicit EncoderQSV(EncoderQSV &other) = delete;

        ~EncoderQSV() Q_DECL_OVERRIDE;

        bool isPaused() Q_DECL_OVERRIDE;

        qint64 registerCallback(IPacketSink *packetSink, int8_t type) Q_DECL_OVERRIDE;

        qint64 unregisterCallback(IPacketSink *packetSink) Q_DECL_OVERRIDE;

        void run() Q_DECL_OVERRIDE;

        EncoderQSV &operator=(const EncoderQSV &other) = delete;

        EncoderQSV &operator=(EncoderQSV &&other) noexcept;

    public slots:

        Q_INVOKABLE int init() Q_DECL_OVERRIDE;

        Q_INVOKABLE int deinit() Q_DECL_OVERRIDE;

        Q_INVOKABLE int start() Q_DECL_OVERRIDE;

        Q_INVOKABLE int stop() Q_DECL_OVERRIDE;

        Q_INVOKABLE void pause(bool pause) Q_DECL_OVERRIDE;

        Q_INVOKABLE int init(IFrameSource *source, AVRational framerate, int64_t duration) Q_DECL_OVERRIDE;

        Q_INVOKABLE int deinit(IFrameSource *source) Q_DECL_OVERRIDE;

        Q_INVOKABLE int start(IFrameSource *source) Q_DECL_OVERRIDE;

        Q_INVOKABLE int stop(IFrameSource *source) Q_DECL_OVERRIDE;

        Q_INVOKABLE void pause(IFrameSource *source, bool paused) Q_DECL_OVERRIDE;

        Q_INVOKABLE void onFrame(IFrameSource *source, AVFrame *frame, int64_t frameDuration, AVBufferRef *pDeviceCtx) Q_DECL_OVERRIDE;

    signals:

        void started() Q_DECL_OVERRIDE;

        void stopped() Q_DECL_OVERRIDE;

        void paused(bool pause) Q_DECL_OVERRIDE;

        void frameProcessingStarted(qint64 pts, qint64 duration) override;

        void frameProcessingFinished(qint64 pts, qint64 duration) override;

    protected:
        [[maybe_unused]] explicit EncoderQSV(EncoderQSVPrivate &p);

        EncoderQSVPrivate *d_ptr;

    };
}

#endif //LIBAVQT_ENCODERQSV_H
